using Ato.CD.Inbound.Shared;
using Ato.EN.IntegrationServices.CodeGenerationPSS;
using Ato.EN.IntegrationServices.CodeGenerationTRTAMI;
using DataContracts;
using System.Collections.Generic;
using System.Linq;

namespace Ato.CD.Inbound.TRTAMI202502
{
    public class CrossFormValidatorPSS : ICrossFormValidator
    {
        private PSS2018 childDocument { get; set; }

        public List<ProcessMessageDocument> response { get; private set; }

        private TRTAMI2025 parentDocument { get; set; }

        public CrossFormValidatorPSS(TRTAMI2025 parent, object child)
        {
            childDocument = (PSS2018)child;
            parentDocument = parent;
            response = new List<ProcessMessageDocument>();
        }

        public List<ProcessMessageDocument> ValidateCrossFormRules()
        {
            VRATOPSS000035();
            VRATOPSS000036();

            return response;
        }

        #region Validation Rules

        #region VR.ATO.PSS.000035

        /*  VR.ATO.PSS.000035
        Payment Type not applicable to Non-individual income tax return

        Legacy Rule Format:
            IF PARENT RETURN <> “IITR” AND ([PSS20] <> NULL OR [PSS21] <> NULL OR [PSS31] <> NULL OR [PSS24] <> NULL OR [PSS25] <> NULL OR [PSS32] <> NULL)
              RETURN VALIDATION MESSAGE
            ENDIF

        Technical Business Rule Format:
            PARENT <> 'IITR' AND (^PSS20 <> NULL OR ^PSS21 <> NULL OR ^PSS31 <> NULL OR ^PSS24 <> NULL OR ^PSS25 <> NULL OR ^PSS32 <> NULL)
        */
        public void VRATOPSS000035()
        {
            bool assertion = childDocument.RPPayerPSSeqNumContextCollection != null && childDocument
                .RPPayerPSSeqNumContextCollection.ToList().Any(p =>
                p.PSS20.HasValue || p.PSS21.HasValue || p.PSS24.HasValue ||
                p.PSS25.HasValue || p.PSS31.HasValue || p.PSS32.HasValue);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.PSS.000035",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Type not applicable to Non-individual income tax return",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000035" } }
                });
            }
        }

        #endregion

        #region VR.ATO.PSS.000036

        /*  VR.ATO.PSS.000036
        Your supplied TFN does not match the TFN supplied on the form it was submitted with

        Legacy Rule Format:
            WHERE IN CONTEXT (RP.Payer.{PSSeqNum})
              IF (RP.Payer.{PSSeqNum}.entity.identifier.TFN <> PARENT RETURN:RP.entity.identifier.TFN)
              RETURN VALIDATION MESSAGE
            ENDIF

        Technical Business Rule Format:
            (^TFN <> PARENT:RP.^TFN2)
        */
        public void VRATOPSS000036()
        {
            bool assertion = childDocument.RPPayerPSSeqNumContextCollection != null && childDocument
                .RPPayerPSSeqNumContextCollection.ToList().Any(p => p.IdentifierTFN != parentDocument.TRTAMI4);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.PSS.000036",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000036" } }
                });
            }
        }

        #endregion

        #endregion
    }
}
